/*
** iplog_dns.c - iplog DNS cache routines.
** Copyright (C) 1999 Odin <odin@numb.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
*/

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <signal.h>
#include <netdb.h>
#include <syslog.h>
#include <string.h>
#include <pthread.h>
#include <sys/socket.h>

#include <config.h>
#include <iplog.h>
#include <iplog_dns.h>
#include <rwlock.h>
#include <iplog_options.h>

static u_char *add_to_cache(u_long addr, u_long hash, u_char *, size_t);

static struct dns_data *dns_table[DNS_MAXSIZE];
static u_int dns_cnt;
rwlock_t dns_lock = RWL_INITIALIZER;
extern u_char flags;

static void dns_cleanup(void *data) {
	struct dns_data *cur = data;

	free(cur->host);
}

/* The locking of the dns table will be greatly improved, this is temporary.. */
void expire_dns(void) {
	size_t i;
	struct dns_data *cur;
	time_t now;

	rwl_writelock(&dns_lock);
	time(&now);
	for (i = 0 ; i < DNS_MAXSIZE ; i++) {
		cur = dns_table[i];
		while (cur != NULL) {
			if (now >= cur->expire) {
				free(cur->host);
				cur = list_delete(cur, (void **) &dns_table[i]);
				--dns_cnt;
			} else
				cur = cur->next;
		}
	}
	rwl_writeunlock(&dns_lock);
}

void destroy_dns_cache(void) {
	size_t i;

	rwl_writelock(&dns_lock);
	for (i = 0 ; i < DNS_MAXSIZE ; i++) {
		if (dns_table[i] != NULL) {
			list_destroy(dns_table[i], dns_cleanup);			
			dns_table[i] = NULL;
		}
	}
	dns_cnt = 0;
	rwl_writeunlock(&dns_lock);
}

u_char *get_dns_cache(u_long addr, u_char *buf, size_t len) {
	u_long hash = HASH(addr, DNS_MAXSIZE);
	bool found = false;
	struct dns_data *cur;

	rwl_readlock(&dns_lock);
	for (cur = dns_table[hash] ; cur != NULL ; cur = cur->next) {
		if (cur->addr == addr) {
			xstrncpy(buf, cur->host, len);
			found = true;
			break;
		}
	}
	rwl_readunlock(&dns_lock);

	if (found == false)
		add_to_cache(addr, hash, buf, len);

	return (buf);
}

static u_char *add_to_cache(u_long addr, u_long hash, u_char *buf, size_t len) {
	struct in_addr in;

	in.s_addr = addr;
	_host_lookup(&in, buf, len);

	rwl_writelock(&dns_lock);
	if (dns_cnt < DNS_MAX_CACHE) {
		struct dns_data temp;

		temp.next = NULL;
		temp.addr = addr;
		temp.expire = time(NULL) + DNS_TIMEOUT;
		temp.host = xstrdup(buf);
		list_prepend(&temp, (void **) &dns_table[hash], sizeof(temp));
		++dns_cnt;
	}
	rwl_writeunlock(&dns_lock);

	return (buf);
}

/* vim:ts=4:sw=8:tw=0 */
