/*
** iplog_syslog.c - iplog logging mechanism. 
** Copyright (C) 1999 Odin <odin@numb.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
*/

#define _GNU_SOURCE
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <syslog.h>
#include <fcntl.h>
#include <time.h>
#include <string.h>
#include <errno.h>
#include <pthread.h>

#include <config.h>
#include <iplog.h>

#define LOG_SIZE 1024

extern char *logfile;

static struct log_data {
	time_t expire;
	u_long times;
	int fd;
	u_char *last_message;
} log;

pthread_mutex_t log_mutex = PTHREAD_MUTEX_INITIALIZER;

void mysyslog(int pri, const char *fmt, ...) {
	time_t now;
	va_list ap;
	u_char *p;

	va_start(ap, fmt);
#ifdef HAVE_VASPRINTF
	vasprintf((char **) &p, fmt, ap);
#else
	p = xmalloc(LOG_SIZE);
	vsnprintf(p, LOG_SIZE, fmt, ap);
#endif
	va_end(ap);

	if (logfile == NULL) {
		pthread_mutex_lock(&log_mutex);
		syslog(pri, p);
		pthread_mutex_unlock(&log_mutex);
		free(p);
		return;
	}
	time(&now);

	pthread_mutex_lock(&log_mutex);
	if (!strcmp(p, log.last_message)) {
		if (now >= log.expire) {
			dprintf(log.fd, "%.15s last message repeated %lu times\n",
				ctime(&now) + 4, log.times);
			log.times = 0;
			log.expire = time(NULL) + 60;
		}

		log.times++;
	} else {
		if (log.times > 0) {
			dprintf(log.fd, "%.15s last message repeated %lu times\n",
				ctime(&now) + 4, log.times);
			log.times = 0;
		}

		dprintf(log.fd, "%.15s %s\n", ctime(&now) + 4, p);
		log.expire = time(NULL) + 60;
		free(log.last_message);
		log.last_message = p;
	}
	pthread_mutex_unlock(&log_mutex);
}

void myopenlog(const char *name, int option, int facility) {
	int fd;

	if (logfile == NULL) {
		openlog(name, option, facility);
		return;
	}

	log.expire = 0;
	log.times = 0;
	log.last_message = xmalloc(1);
	*log.last_message = '\0';	

	fd = open(logfile, O_WRONLY | O_CREAT | O_APPEND, 0644);

	if (fd == -1) {
		fatal("Fatal: Unable to open \"%s\" for writing: %s.",
			logfile, strerror(errno));
	}

	log.fd = fd;
	mysyslog(PRIORITY, "iplog started.");
}

void mycloselog(void) {
	if (logfile == NULL)
		closelog();
	else {
		time_t now;

		pthread_mutex_lock(&log_mutex);

		time(&now);
		if (log.times > 0) {
			dprintf(log.fd, "%.15s last message repeated %lu times\n",
				ctime(&now) + 4, log.times);
        }
		close(log.fd);

		log.fd = -1;
		log.expire = 0;
		log.times = 0;
		log.last_message = xmalloc(1);
		*log.last_message = '\0';	

		pthread_mutex_unlock(&log_mutex);
	}
}

/* vim:ts=4:sw=8:tw=0 */
